unit MainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, ShellApi;

type
  TForm1 = class(TForm)
    LeftChannelFilenameEdit: TEdit;
    Label1: TLabel;
    RightChannelFilenameEdit: TEdit;
    Label2: TLabel;
    SaveDialog1: TSaveDialog;
    CombineButton: TButton;
    UpDown1: TUpDown;
    LeftOpenButton: TSpeedButton;
    RightOpenButton: TSpeedButton;
    OpenDialog1: TOpenDialog;
    Label4: TLabel;
    OutputFilenameEdit: TEdit;
    Label3: TLabel;
    SpeedButton1: TSpeedButton;
    ProgressLabel: TLabel;
    procedure UpDown1Click(Sender: TObject; Button: TUDBtnType);
    procedure CombineButtonClick(Sender: TObject);
    procedure LeftOpenButtonClick(Sender: TObject);
    procedure RightOpenButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Label3Click(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
  protected
    procedure WMDROPFILES (var Msg: TMessage); message WM_DROPFILES;
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TwFormatTag = record
    wFormatTag: SmallInt;
    nChannels: SmallInt;
    nSamplesPerSec: Integer;
    nAvgBytesPerSec: Integer;
    nBlockAlign: SmallInt;
    wBitsPerSample: SmallInt;
    dataChunkOffset: Integer;
    dataChunkSize: Integer;
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

// http://www.delphipages.com/tips/copyview.cfm?ID=95
procedure TForm1.WMDROPFILES(var Msg: TMessage);
var
  i, amount, size: integer;
  Filename: PChar;
  FilenameExtension: string;
  FileNameArray: array[0..1] of string;
begin
  inherited;
  Filename := '';
  Amount := DragQueryFile(Msg.WParam, $FFFFFFFF, Filename, 255);
  case Amount of
    1:
      begin
        i := 0;
        size := DragQueryFile(Msg.WParam, i, nil, 0) + 1;
        Filename := StrAlloc(size);
        DragQueryFile(Msg.WParam, i, Filename, size);
        FilenameExtension := ExtractFileExt(Filename);
        if (FilenameExtension <> '.wav') then
          begin
            ShowMessage(Filename + ' is not a .wav file - aborting');
          end
        else
          begin
            if (LeftChannelFilenameEdit.Text = '') then
              LeftChannelFilenameEdit.Text := StrPas(Filename)
            else if ((LeftChannelFilenameEdit.Text <> '') and (RightChannelFilenameEdit.Text <> '')) then
              LeftChannelFilenameEdit.Text := StrPas(Filename)
            else
              RightChannelFilenameEdit.Text := StrPas(Filename);

            if (OutputFilenameEdit.Text = '') then
              OutputFilenameEdit.Text := ChangeFileExt(ChangeFileExt(Filename, ''), '.stereo.wav');
          end;
        StrDispose(Filename);
      end;

    2:
      begin
        for i := 0 to (Amount - 1) do
          begin
            size := DragQueryFile(Msg.WParam, i, nil, 0) + 1;
            Filename := StrAlloc(size);
            DragQueryFile(Msg.WParam, i, Filename, size);
            FileNameArray[i] := Filename;
            FilenameExtension := ExtractFileExt(Filename);
            if (FilenameExtension <> '.wav') then
              begin
                ShowMessage(Filename + ' is not a .wav file - aborting');
                exit;
              end;
            StrDispose(Filename);
          end;

        if ((ChangeFileExt(ChangeFileExt(ExtractFileName(FileNameArray[0]), ''), '.L.wav') = ExtractFileName(FileNameArray[0])) and (ChangeFileExt(ChangeFileExt(ExtractFileName(FileNameArray[1]), ''), '.R.wav') = ExtractFileName(FileNameArray[1]))) then
          begin
            // first filename is left channel, second is right channel, filenames match
            LeftChannelFilenameEdit.Text  := FileNameArray[0];
            RightChannelFilenameEdit.Text := FileNameArray[1];
            OutputFilenameEdit.Text := ChangeFileExt(ChangeFileExt(FileNameArray[0], ''), '.stereo.wav');
            CombineButton.Click();
          end
        else if ((ChangeFileExt(ChangeFileExt(ExtractFileName(FileNameArray[0]), ''), '.R.wav') = ExtractFileName(FileNameArray[0])) and (ChangeFileExt(ChangeFileExt(ExtractFileName(FileNameArray[1]), ''), '.L.wav') = ExtractFileName(FileNameArray[1]))) then
          begin
            // first filename is right channel, second is left channel, filenames match
            RightChannelFilenameEdit.Text := FileNameArray[0];
            LeftChannelFilenameEdit.Text  := FileNameArray[1];
            OutputFilenameEdit.Text := ChangeFileExt(ChangeFileExt(FileNameArray[0], ''), '.stereo.wav');
            CombineButton.Click();
          end
        else
          begin
            // filenames don't match in any known way
            LeftChannelFilenameEdit.Text  := FileNameArray[0];
            RightChannelFilenameEdit.Text := FileNameArray[1];
            OutputFilenameEdit.Text := ChangeFileExt(ChangeFileExt(FileNameArray[0], ''), '.stereo.wav');
          end;
      end;
    else
      ShowMessage('Cannot combine more than 2 files!');
  end;

  if ((LeftChannelFilenameEdit.Text = '') or (RightChannelFilenameEdit.Text = '')) then
    begin
      OutputFilenameEdit.Text := '';
      UpDown1.Enabled := false;
      CombineButton.Enabled := false;
    end
  else
    begin
      UpDown1.Enabled := true;
      CombineButton.Enabled := true;
    end;
  SaveDialog1.FileName := OutputFilenameEdit.Text;

  DragFinish(Msg.WParam);
end;

procedure TForm1.UpDown1Click(Sender: TObject; Button: TUDBtnType);
var
  TempFilename: string;
begin
  TempFilename                  := LeftChannelFilenameEdit.Text;
  LeftChannelFilenameEdit.Text  := RightChannelFilenameEdit.Text;
  RightChannelFilenameEdit.Text := TempFilename;
end;

procedure TForm1.LeftOpenButtonClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
    LeftChannelFilenameEdit.Text := OpenDialog1.FileName;
  if ((LeftChannelFilenameEdit.Text <> '') and (RightChannelFilenameEdit.Text <> '')) then
    begin
      UpDown1.Enabled := true;
      CombineButton.Enabled := true;
    end;
end;

procedure TForm1.RightOpenButtonClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
    RightChannelFilenameEdit.Text := OpenDialog1.FileName;
  if ((LeftChannelFilenameEdit.Text <> '') and (RightChannelFilenameEdit.Text <> '')) then
    begin
      UpDown1.Enabled := true;
      CombineButton.Enabled := true;
    end;
end;




// http://www.delphicorner.f9.co.uk/articles/misc12.htm
function GetwFormatTag(WaveFile: string): TwFormatTag;
var
  groupID: array[0..3] of char;
  riffType: array[0..3] of char;

  WaveFormat: TwFormatTag;
  Stream: TFileStream;
  // chunk seeking function,
  // -1 means: chunk not found

function GotoChunk(ID: string; SizeOrPosition: string): Integer;
var
  chunkID: array[0..3] of char;
  chunkSize: Integer;
begin
  Result := -1;

  with Stream do
    begin
      // index of first chunk
      Position := 12;
      repeat
        // read next chunk
        Read(chunkID, 4);
        Read(chunkSize, 4);
        if chunkID <> ID then
          begin
            // skip chunk
            Position := Position + chunkSize;
          end;
      until (chunkID = ID) or (Position >= Size);
      if chunkID = ID then
        begin
          // chunk found,
          // return chunk size

          if (SizeOrPosition = 'size') then
            Result := chunkSize
          else
            Result := Position;
        end;
    end;
  end;

  begin
    Stream := TFileStream.Create(WaveFile, fmOpenRead or fmShareDenyNone);
    with Stream do
      try
        Read(groupID, 4);
        Position := Position + 4; // skip four bytes (file size)
        Read(riffType, 4);
   
        if (groupID = 'RIFF') and (riffType = 'WAVE') then
          begin
            // search for format chunk
            if GotoChunk('fmt ', 'size') <> -1 then
              begin
                // found it
                Read(WaveFormat.wFormatTag, 2);
                Read(WaveFormat.nChannels, 2);
                Read(WaveFormat.nSamplesPerSec, 4);
                Read(WaveFormat.nAvgBytesPerSec, 4);
                Read(WaveFormat.nBlockAlign, 2);
                Read(WaveFormat.wBitsPerSample, 2);

                WaveFormat.dataChunkOffset := GotoChunk('data', 'position');
                WaveFormat.dataChunkSize   := GotoChunk('data', 'size');

                Result := WaveFormat;
              end;
          end;
      finally
        Free;
      end;
end;


// http://www.techweb.rfa.org/docs/RIFFWAVE.htm
procedure TForm1.CombineButtonClick(Sender: TObject);
var
  WaveFormatLeft, WaveFormatRight: TwFormatTag;
  blah, i: Integer;
  SamplePoint, SamplesToCopyPerChannel: Integer;
  BytesPerSample: Byte;
  FileHandleOut, FileHandleLeft, FileHandleRight: Integer;
begin
  ProgressLabel.Font.Color := clRed;
  ProgressLabel.Caption := 'starting...';

  if (OutputFilenameEdit.Text = '') then
    begin
      ProgressLabel.Caption := 'error';
      CombineButton.Enabled := false;
      exit;
    end
  else if (FileExists(OutputFilenameEdit.Text)) then
    begin
      SaveDialog1.FileName := OutputFilenameEdit.Text;
      if (SaveDialog1.Execute) then
        OutputFilenameEdit.Text := SaveDialog1.FileName
      else
        begin
          ProgressLabel.Caption := 'error';
          exit;
        end;
    end;


  WaveFormatLeft  := GetwFormatTag(LeftChannelFilenameEdit.Text);
  WaveFormatRight := GetwFormatTag(RightChannelFilenameEdit.Text);

  ProgressLabel.Caption := 'error';
  if (WaveFormatLeft.wFormatTag <> 1) then
    ShowMessage('Left channel not PCM data - aborting')
  else if (WaveFormatRight.wFormatTag <> 1) then
    ShowMessage('Right channel not PCM data - aborting')
  else if (WaveFormatLeft.nChannels <> 1) then
    ShowMessage('Left channel not mono - aborting')
  else if (WaveFormatRight.nChannels <> 1) then
    ShowMessage('Right channel not mono - aborting')
  //else if (WaveFormatLeft.wBitsPerSample <> 16) then
  //  ShowMessage('Left channel not 16-bit - aborting')
  //else if (WaveFormatRight.wBitsPerSample <> 16) then
  //  ShowMessage('Right channel not 16-bit - aborting')
  else if (WaveFormatLeft.nSamplesPerSec <> WaveFormatRight.nSamplesPerSec) then
    ShowMessage('Left and Right channels have different sample rates - aborting')
  else if (WaveFormatLeft.nBlockAlign <> WaveFormatRight.nBlockAlign) then
    ShowMessage('Left and Right have different nBlockAlign values - aborting')
  else if (WaveFormatLeft.dataChunkSize <> WaveFormatRight.dataChunkSize) then
    ShowMessage('Left and Right have different data chunk sizes - aborting')
  else
    begin
      ProgressLabel.Font.Color := clBlue;
      ProgressLabel.Caption := 'processing...';
      ProgressLabel.Repaint;
      BytesPerSample := (WaveFormatLeft.wBitsPerSample div 8);

      FileHandleOut   := FileCreate(OutputFilenameEdit.Text);
      FileHandleLeft  := FileOpen(LeftChannelFilenameEdit.Text, fmOpenRead);
      FileHandleRight := FileOpen(RightChannelFilenameEdit.Text, fmOpenRead);
      try
        FileWrite(FileHandleOut, 'RIFF', 4);
        blah := 44 + WaveFormatLeft.dataChunkSize + WaveFormatRight.dataChunkSize;
        FileWrite(FileHandleOut,   blah, 4);
        FileWrite(FileHandleOut, 'WAVE', 4);
        FileWrite(FileHandleOut, 'fmt ', 4);
        blah := 16;
        FileWrite(FileHandleOut,   blah,                        4); // size of 'fmt ' chunk
        FileWrite(FileHandleOut, WaveFormatLeft.wFormatTag,     2); // wFormatTag
        blah := 2;
        FileWrite(FileHandleOut,   blah,                        2); // nChannels
        FileWrite(FileHandleOut, WaveFormatLeft.nSamplesPerSec, 4); // nSamplesPerSec
        blah := WaveFormatLeft.nAvgBytesPerSec * 2;
        FileWrite(FileHandleOut,   blah,                        4); // nAvgBytesPerSec
        FileWrite(FileHandleOut, WaveFormatLeft.nBlockAlign,    2); // nBlockAlign
        FileWrite(FileHandleOut, WaveFormatLeft.wBitsPerSample, 2); // wBitsPerSample
        FileWrite(FileHandleOut, 'data', 4);
        blah := WaveFormatLeft.dataChunkSize + WaveFormatRight.dataChunkSize;
        FileWrite(FileHandleOut,   blah, 4);

        FileSeek(FileHandleLeft,  WaveFormatLeft.dataChunkOffset,  0);
        FileSeek(FileHandleRight, WaveFormatRight.dataChunkOffset, 0);
        SamplesToCopyPerChannel := WaveFormatLeft.dataChunkSize div BytesPerSample;
        for i := 0 to SamplesToCopyPerChannel do
          begin
            if ((i mod 1000) = 0) then
              begin
                ProgressLabel.Caption := 'processing [' + IntToStr(round((i / SamplesToCopyPerChannel) * 100)) + '%]...';
                ProgressLabel.Repaint;
              end;
            FileRead(FileHandleLeft,  SamplePoint, BytesPerSample);
            FileWrite(FileHandleOut,  SamplePoint, BytesPerSample);
            FileRead(FileHandleRight, SamplePoint, BytesPerSample);
            FileWrite(FileHandleOut,  SamplePoint, BytesPerSample);
          end;

      finally
        FileClose(FileHandleOut);
        FileClose(FileHandleLeft);
        FileClose(FileHandleRight);

        LeftChannelFilenameEdit.Text  := '';
        RightChannelFilenameEdit.Text := '';
        OutputFilenameEdit.Text       := '';

        ProgressLabel.Font.Color := clGreen;
        ProgressLabel.Caption := 'done - waiting...';

      end;
    end;

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  DragAcceptFiles(Form1.Handle, true);
end;

procedure TForm1.Label3Click(Sender: TObject);
begin
  ShellExecute(Application.Handle,
             PChar('open'),
             PChar('http://www.silisoftware.com'), 
             PChar(0),
             nil,
             SW_NORMAL);
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
begin
  ShowMessage('What does DualMono2Stereo do?' + #13#10#13#10 + 'DualMono2Stereo takes two mono WAV files and interleaves them into one stereo WAV file.' + #13#10#13#10#13#10 + 'Where can I download the latest version?' + #13#10#13#10 + 'http://www.silisoftware.com' + #13#10#13#10#13#10 + 'Who can I complain to?' + #13#10#13#10 + 'info@silisoftware.com' + #13#10#13#10#13#10 + 'Is this software free?' + #13#10#13#10 + 'Yes, this software is free to use.');
end;

end.
